#!/bin/ksh
#  
# cdsevent script
# NAME        Version           Date
# C Stone     0.1		15/01/2004
#  
#####################################################################
#
# Name:    cdsevent
#
# Purpose: Logs event summary information generated by Event Response resource
#          manager to a specified log file. The log file is an event summary
#          log file that containe a one or two line per problem report.
#          This log file can be tailed and is highlighted with colours.
# Log Colours:
#          The Day ,Date and Time always appear at the start of every entry.
#          The colour of this part are governed by the Message Type:
#          The remaining part of the message can be a different colour if required.
#          GPFS Waiters error messages will be highlighted in Blue.
#          The colours used are as follows:
#           - Day Date and Time at the start of the line 
#               Message Type        Colours
#               ============        =======
#             - Critical            Red Bold
#             - Warnings            Red 
#             - Information         Magenta
#
#          All 'Rearm' Events will be coloured Green regardless of Message type.
# Inputs:  Name of the file where event information is logged.
#          An absolute path for the logfile operand should be specified.  
#
#####################################################################

generate_message () {
# Here is that data that is available
#${ConditionName}: $ERRM_COND_NAME
#${Severity}: $COND_SEVERITY
#${EventType}: $TYPE
#${Expression}: $ERRM_EXPR

#${ResourceName}: $ERRM_RSRC_NAME
#${ResourceClass}: $ERRM_RSRC_CLASS_PNAME
#${DataType}: $ERRM_DATA_TYPE
#${DataValue}: $ERRM_VALUE
#${NodeName}: $ERRM_NODE_NAME
#${NodeNameList}: $ERRM_NODE_NAMELIST
#${RsrcType}: $ERRM_RSRC_TYPE

#Here is the format of the message....
#TIME         : ${Severity}     : ${ConditionName} ${DataType}
#                          Host: ${NodeName}     : ${ResourceName}   : ${Expression}: 


        cat  <<EOF
$date_colour${EventTime}:$colour $COND_SEVERITY  : $ERRM_COND_NAME $MSG
                           Host=$ERRM_NODE_NAME  , STRING=$ERRM_VALUE ,RES_NAME=$ERRM_RSRC_NAME$plain
EOF
        
}

# main()

PERL=/usr/sbin/rsct/perl5/bin/perl
# Instead of using ctdspmsg, we directly call dspmsg to improve the
# performance. 
# Need to ensure the set id and message id are correct.  
#CTMSG=/usr/sbin/rsct/bin/ctdspmsg
CTMSG=/usr/bin/dspmsg
MSGMAPPATH=/usr/sbin/rsct/msgmaps
export MSGMAPPATH


ConditionName=`$CTMSG -s 5 IBM.ERrm.cat 2 'Condition Name'`
Severity=`$CTMSG -s 5 IBM.ERrm.cat 3 'Severity'`
EventType=`$CTMSG -s 5 IBM.ERrm.cat 4 'Event Type'`
Expression=`$CTMSG -s 5 IBM.ERrm.cat 5 'Expression'`

ResourceName=`$CTMSG -s 5 IBM.ERrm.cat 6 'Resource Name'`
ResourceClass=`$CTMSG -s 5 IBM.ERrm.cat 8 'Resource Class'`
DataType=`$CTMSG -s 5 IBM.ERrm.cat 9 'Data Type'`
DataValue=`$CTMSG -s 5 IBM.ERrm.cat 10 'Data Value'`
NodeName=`$CTMSG -s 5 IBM.ERrm.cat 7 'Node Name'`
NodeNameList=`$CTMSG -s 5 IBM.ERrm.cat 14 'Node NameList'`
RsrcType=`$CTMSG -s 5 IBM.ERrm.cat 15 'Resource Type'`
Usage=`$CTMSG -s 5 IBM.ERrm.cat 11 'Usage'`
InvalidDir=`$CTMSG -s 5 IBM.ERrm.cat 12 'invalid directory'`

# Get the Colour codes
. /csminstall/tools/monitor/Colour.Codes
#

##################################################################
# Convert Severity into colour codes.
##################################################################
case $ERRM_COND_SEVERITYID in

   0 ) COND_SEVERITY=`$CTMSG -s 5 IBM.ERrm.cat 19 'Informational'`
       colour=$magenta 
       plain=$black
       ;;
   1 ) COND_SEVERITY=`$CTMSG -s 5 IBM.ERrm.cat 20 'Warning'`
       colour=$red
       plain=$black
       ;;
   2 ) COND_SEVERITY=`$CTMSG -s 5 IBM.ERrm.cat 21 'Critical'`
       colour=$red$bold
       plain=$black$boldoff
       ;;

esac

##################################################################
# Convert EventType
#  If the event type is 'Rearm' then make the whole message Green
##################################################################
case $ERRM_TYPEID in

   0 ) TYPE=`$CTMSG -s 5 IBM.ERrm.cat 17 'Event'`
       MSG="has occurred"
       date_colour=$colour
       ;;
   1 ) TYPE=`$CTMSG -s 5 IBM.ERrm.cat 18 'Rearm event'`
       MSG="has reset"
       date_colour=$green
       colour=$green
       ;;

esac

##################################################################
#  Process the command line options - logfile name
##################################################################
while getopts ":h" opt
do
  case $opt in

    h ) print "${Usage} : `basename $0` [-h] logfile "
        exit 0;;

    ? ) print "${Usage} : `basename $0` [-h] logfile "
        exit 3;;
  esac
done


##################################################################
# Now do the special message colour options
# Theese include:
#  GPFS Waiters messages in Blue
##################################################################
case ${ConditionName} in

   GPFS_prob_waiters )
       colour=$blue$bold
       plain=$black$boldoff
       ;;

esac

##################################################################
# Check if a required logfile is specified
##################################################################
if [[ $# -lt 1 ]]; then
   print -u2 "${Usage} : `basename $0` [-h] logfile "
   exit 1
fi

# Check if the logfile path is valid
fileName=$1
pathName=${fileName%/*}

if [[ -z $pathName ]]; then
   pathName=/
fi

if [[ ! -d $pathName ]]; then
   print -u2 "${InvalidDir} "
   exit 2
fi

##################################################################
# Create the Day -Date -Time for the log
##################################################################
# convert time string
seconds=${ERRM_TIME%,*}

EventTime=$(seconds=$seconds $PERL -e \
'
use POSIX qw(strftime);
print strftime("%a %d/%m/%y %T", localtime($ENV{seconds}) );  

'
)

##################################################################
# Write the message to the summary log file
##################################################################
data=$(generate_message) ofile=$1   \
$PERL -e \
'
use Fcntl qw(:DEFAULT :flock);
open(ofh,">>/tmp/errmlogevent.lck");
flock(ofh,LOCK_EX);

# CDS Do not use the alog as this is fixed length
#$uname=`uname`;  chomp $uname;
# alog only exists on aix
#if ("$uname" eq "AIX")
#{
# 
# `echo "\$data" | alog -q -f "\$ofile" -s 65536` ;
#}
#else
#{
  open(ofh2,">>$ENV{ofile}");  
  print ofh2 "$ENV{data}\n" ;
  close(ofh2);
#}

close(ofh);  

'
