#!/usr/bin/perl
package CSMSIS;

use strict;
use warnings;
use Carp; 
use File::Copy;

use lib ($ENV{'CSM_PM'} || '/opt/csm/pm');
use FindBin qw($Bin);
use lib "$Bin";

use NodeUtils;

use base "Exporter";
our @ISA		= qw(Exporter);
our @EXPORT		= qw(get_SIS_config SIS_setup_golden_client);
our @EXPORT_OK	= qw(freshen parse_pipey_hash hash_to_pipey run_remote_command);

=head1 NAME

CSMSIS.pm

=head1 SYNOPSIS

A module for interfacing CSM with SIS.

=head1 DESCRIPTION

This file contains the following routines.

=cut


1; # we are a HAPPY MODULE



=over 4

=item SIS_get_image

This routine will call si_getimage with appropriate options.
It accepts a reference to a hash of options ($o) which will
determine the behaviour it will exhibit.
This hash must include the following elements:

=over 6

=item $o->{'golden-client'} 

This element should be set to the hostname of a machine which will be
used as the golden client.

=item $o->{image}

This is the name which the image will be saved under.

=back

All other elements are optional.

=cut

sub SIS_get_image($) {
	print "Obtaining an image....\n";
	my ($params) = @_;
	my $command = 'si_getimage' .
				  ' -golden-client ' .	$params->{'golden-client'} .
				  ' -image ' .			$params->{'image'};
	
	$command .= ' -quiet';
	#$command .= ($params->{'verbose'} ? ' --yes' : ' -quiet');
	# We have to do it -quiet or it asks too many questions! No --yes.
	# Consider if perhaps we should pipe it, yes | si-getimage ....
	
	$command .= ' --no-listing'	unless $params->{'listing'};	
	$command .= ' --post-install '  . ($params->{'post-install'}  || 'reboot');
	$command .= ' --ip-assignment ' . ($params->{'ip-assignment'} || 'static');
	$params->{'exclude'}     and ($command .= ' --exclude='      . $params->{'exclude'});
	$params->{'ssh-user'}     and ($command .= ' --ssh-user '     . $params->{'ssh-user'});
	$params->{'directory'}     and ($command .= " --directory '"   . $params->{'directory'} . "'");
	$params->{'exclude-file'}   and ($command .= ' --exclude-file ' . $params->{'exclude-file'});
	$params->{'update-script'}   and ($command .= ' --update-script YES');
	$params->{'autodetect-disks'} and ($command .= ' --autodetect-disks');
	# TODO??? excludes,  autodetect disks
	
	print "Running command: $command\n";
	my @lines = NodeUtils->runcmd($command);
	print join("\n",@lines);
	print "\n";
}

sub run_remote_command($$){
	my ($client, $command) = @_;
	my $dsh_command = 'dsh -s -n ' . $client . ' ' . $command;
	print "Running command: $dsh_command\n";
	my @lines = NodeUtils->runcmd($dsh_command, "W");
	print join("\n",@lines);
	print "\n";

}


sub SIS_setup_golden_client($) {
	my ($params) = @_;	
	my 	$command  = 'si_prepareclient --server `mgmtsvr`';
	
	$command .= ' --no-uyok'  if $params->{'no-uyok'};
	$command .= 
		($params->{'verbose'} ? ' --yes' : ' --quiet');
	$ENV{DSH_PATH}='/usr/bin:/bin:/usr/sbin:/sbin:/opt/csm/bin';
	
	run_remote_command($params->{'golden-client'}, "'$command &'");
	# note the ''s and the & (the latter keeps rsync running after dsh exits
}



sub shutdown_rsyncd($){
	my ($golden_client) = @_;
	my $command = 'killall -HUP rsync';
	# this searches for the daemon listening on port 873, gets its pid, and kills it.
	run_remote_command($golden_client,$command);
}




sub make_autoinstall_script($){
	die "Please don't make_autoinstall_script yet.";
	my ($params) = @_;
	my
	$command  = 'si_mkautoinstallscript' .
			    ' --image ' . $params->{image};
	$command .= ' --quiet'		unless $params->{'verbose'};	
	$command .= ' --no-listing' unless $params->{'listing'};
    $command .= ' --post-install '  . ($params->{'post-install'}  || 'reboot');
    $command .= ' --ip-assignment ' . ($params->{'ip-assignment'} || 'static');
	$command .= ' --config-file ' . $params->{'config-file'} if $params->{'config-file'};
	$command .= ' --force';
}


=item launch_daemons

This routine launches the /etc/init.d/systemimager-server-* installation daemons.

=cut

sub launch_daemons {
	print "Launching SIS installation daemons\n";
	# these are needed for the install
	NodeUtils->runcmd("/etc/init.d/systemimager-server-rsyncd start");
	NodeUtils->runcmd("/etc/init.d/systemimager-server-netbootmond start");
	NodeUtils->runcmd("/etc/init.d/systemimager-server-flamethrowerd start")
		if -x '/etc/init.d/systemimager-server-flamethrowerd'
		   and not -e '/var/run/systemimager-server-flamethrowerd.pid';
		# this one whines about being started if it's already running

};



sub freshen($$) { # Make sure something is up to date.
	my ($src, $dst) = @_;
	{
		my @stats = stat($src) or croak "Could not stat $src: $!";
		next	if !-e   $dst;
		die		if !-e	 $src;
		return	if (stat($dst))[9] < $stats[9];
	}	# (stat())[9] should is a timestamp.

	unlink($dst);
	copy($src => $dst) or croak "Could not copy $src to $dst: $!";
}


sub parse_pipey_hash($) {
	shift;
	my %hash = split /\|:\|/;
	return \%hash;

	# You could make an argument for this function being somewhere else.
	# However, while it might be handy for something like ServerUtils or
	# such... well, they don't seem to be using it, and regardless of
	# whether or not they use it, we are going to use this format over here.
}

sub hash_to_pipey($){
	my ($hash) = @_;
	return join '|:|', @{$hash};
	# access the hashref as an array
}



=item get_SIS_config

This function should be used to determine the value of a SIS
configuration directive. If the SIS app-config module has not
been loaded, this routine will load it.

This routine should be used, instead of a direct SIS interface,
to avoid compile-time dependencies on SIS code (since this
module might be accessed from another CSM function which may
have modes of operation which do not rely on SIS).

=cut

sub get_SIS_config($) {
		local @INC;
		push @INC, "/usr/lib/systemimager/perl";
		push @INC, "/usr/lib/perl5/site_perl";
		require SystemImager::Config	# load it...
				unless $::main::config; # if we don't have it yet
		return $::main::config->get(shift);
}


sub setup_services {
	print "Setting up services\n";
	my ($o, $nodehash) = @_;
	
	my %cluster_info_hash = (); # = (
	#	"logfile_DCsyncserver"		=> $o->{logfile},
	#	"logfile_DCenableservices"	=> $o->{logfile},
	#	"logfile_DCsetupinstaller"	=> $o->{logfile},
	#	); # seems to hold stuff about logfiles
	my $nodeinfo_hash = 
		ServerUtils::make_nodeinfo_hash($nodehash);
	my ($ref_nodehash,
		$ref_ishash,
		$ref_istonodehash) =
		ServerUtils::getTargetNodeHashandInstallServerHash($nodehash);
	my $ISinfo_hash =
		ServerUtils::make_nodeinfo_hash($ref_ishash);

	print "Callin' API business\n";

	DCAPI->DCenableservices(
		\%cluster_info_hash,
		$nodeinfo_hash,
		$ISinfo_hash,
		0, $o->{'verbose'}
		);
}



sub setup_node{
	my $attr = shift;
	my $o = shift;
	my $template = shift;
	my %props = split(/\|:\|/ => $attr->{Properties});

	# TODO load some Properties from the template-file	
	while(my ($k,$v) = each(%$template)){
		$props{$k} = $v;
		}

	my $scriptdir = get_SIS_config('autoinstall_script_dir') || 
					'/var/lib/systemimager/scripts';

	my $longhostname = $attr->{Hostname}		or die "No hostname set for node";
	my $hostname  =	(split(/\./ => $longhostname))[0];
	my $imagename =	$props{InstallImageName}	or die "No InstallImageName property set for $hostname";
	my $arch = $attr->{InstallPkgArchitecture}	or die "InstallPkgArchitecture not set for $hostname";

	$attr->{ShortHostname} = $hostname;

	print "Setting up node: $hostname\n";

	{	# autoinstallscript setup
		# $hostname.sh will run when the node boots
		my $scriptname	  = $props{InstallScriptName} || "$imagename.master";
		$scriptname = subHash($scriptname, $attr);
		$scriptname = subHash($scriptname, \%props);
	
		my $masterscript = "$scriptdir/$scriptname";
		my $targetscript = "$scriptdir/$hostname.sh";
		-f $masterscript or die "No $scriptname script exists in $scriptdir";

		unlink($targetscript); # or-die not required
							   # it might legitimately not-be-there

		# Set up the autoinstall script to point to a master
		# this is what si_addclients does
		# we can do this the hard way or the easy way
		############################## STRIKE THAT
		# we do it the hard way.

		my @variables;
		# should we have an InstallVariables property here?
		# How would we deal with multiple variables? We're already |:|ed out.
		
		if( ($props{InstallTransferMethod} || '') eq 'flamethrower'){
			# Strictly speaking, it's a little late to be doing all this
			# since by the time we're at the autoinstall script, we've
			# already rsynced over a number of things - but still, the
			# Flamethrower stuff will at least get used for the BIG things
			# gigabytes versus megabytes

			# Really, we should be doing this in DHCP though.
			#			option-143 "9000";
			my $portbase = $props{InstallFlamethrowerPortbase} || "9000";
			# shouldn't that really be in $config somewhere? eh.
			# I don't expect that attribute shall get set very much anyway...
			push @variables, "FLAMETHROWER_DIRECTORY_PORTBASE=$portbase";
			}
		my $override = $props{InstallOverrides};
		push @variables => "OVERRIDES=$override" if $override;
		my $variablesToSet = join("\n",@variables);

		open my $tsfile   => ">", $targetscript;
		my $scriptcontents =  <<'EOF';
# Instead of symlinking to the master script...
# do our own stuff (set variables etc) THEN call it

# The first action of an autoinstall script is to source /tmp/variables.txt
# So let's set it there. Easier for debuggin' anyway. And lets us override
# if we care to.
echo '$variablesToSet' >> /tmp/variables.txt

# now source it
. /tmp/variables.txt

# if we're using Flamethrower
# get the dir, since we don't actually have it yet
if [ ! -z \$FLAMETHROWER_DIRECTORY_PORTBASE]; then
	. /etc/init.d/functions
	get_flamethrower_directory
fi

cd /scripts
chmod 775 $scriptname
# just in case 
./$scriptname
EOF
		$scriptcontents =~ s/\$variablesToSet/$variablesToSet/;

		print $tsfile $scriptcontents;
		close $tsfile;
		#} else {
		if(0) {  # the easy way -- BLAH we're not doing this
			symlink($scriptname => $targetscript)
				or die "Could not install $targetscript: $!";
				# note use of the relative path in symlink
				# the absolute path will fail in the install environment
		}	# see now? wasn't that easy?
	}	# okay, autoinstalliness done


	freshen("/tftpboot/sis/$arch/$imagename/initrd.img" => "/tftpboot/sis/$imagename.i");
	

	# Create a PXE config file.
	{	# scope games
		# NOTE THE SINGLE QUOTES
		# we don't want to interpolate %FOO% as a hashtable followed by a %!
		# We've left out the SERIAL option! Hmmph!
		# Well, we're not printing anything HERE... SIS defaults aside...
		my $stub = << 'EOF';
DEFAULT sis
LABEL sis
KERNEL %InstallKernel%
APPEND %InstallKernelAppend%
EOF
		my $console = 'console=%ConsoleSerialDevice%,%ConsoleSerialSpeed%' 
			 if $attr->{ConsoleSerialDevice};

		$stub =~ s/\%InstallKernel\%/$props{InstallKernel}/g;
		$stub =~ s/\%InstallKernelAppend\%/$props{InstallKernelAppend}/g;
		$stub =~ s/\%CONSOLE\%/$console/ if $console;

		$stub = subHash($stub, \%props, 5);
		$stub = subHash($stub, $attr);
			# substitute attributes afterwards
			# properties may reference attributes
			# but vice-versa doesn't work.

=for comment
		$stub =~ s/\%RAMDISKSIZE\%/$ramdisksize/g;
		$stub =~ s/\%RAMDISK\%/$ramdiskspec/g;  # Notice! This comes before IMG/ARCH subs
		$stub =~ s/\%KERNEL\%/$kernel/g;		# and this!
		$stub =~ s/\%APPEND\%/$append/g;		# this too!
		$stub =~ s/\%ARCH\%/$arch/g;
		$stub =~ s/\%IMG\%/$imagename/g;
=cut
		my $pfh; # PXE File Handle
		my $pxefile = "/tftpboot/pxelinux.cfg/$longhostname.sis";
		# XXX Note: existing DCAPI stuff looks for .sis instead of .install.
		# XXX I don't quite know what this means for the future.
		open $pfh => ">", $pxefile or die "Could not write to $pxefile: $!";
		print $pfh $stub; #it's not really a stub anymore now, but eh
		close $pfh;
	}	# send all that stuff there out of scope!



	
}	# and now you're ready to install! yay hooray

sub subHash {
    my $tosub = shift;
	my $hashref = shift;
	my $count = 5;
		# okay. why a count? well, properties may reference each other.
		# %InstallKernelAppend% references %InstallRamdisk% references %InstallImageName%.
		# soo... rather than actually determining when they're done referencing each other
		# (which could very well be never, stupid infinite loops)
		# we'll just substitute five times
		# it should be 'enough for anybody'
		# 
		# now watch this show up on thedailywtf.com. -- tsmw.
	while($count--){
	    while( my ($aName,$aVal) = each(%$hashref) ){
    	    $tosub =~ s/\%$aName\%/$aVal/g if $aName and $aVal;
	    }
	}
    return $tosub;
}
										

# this appears to be the end







=head1 AUTHOR

Thomas Whaples <tswhaple@us.ibm.com

=head1 COPYRIGHT

Copyright 2006, IBM. All rights reserved.


=cut
